/*
 * Trace Recorder for Tracealyzer v4.8.2
 * Copyright 2023 Percepio AB
 * www.percepio.com
 *
 * SPDX-License-Identifier: Apache-2.0
 *
 * Supporting functions for trace streaming, used by the "stream ports" 
 * for reading and writing data to the interface.
 * Existing ports can easily be modified to fit another setup, e.g., a 
 * different TCP/IP stack, or to define your own stream port.
 */

#include <trcRecorder.h>

#if (TRC_USE_TRACEALYZER_RECORDER == 1)

#if (TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_STREAMING)  

// FreeRTOS includes.
#include "FreeRTOS.h"

#include "FreeRTOS_IP.h"
#include "FreeRTOS_Sockets.h"


Socket_t xSocket = NULL;
Socket_t xAcceptSocket = NULL;

struct freertos_sockaddr 	xBindAddress;
struct freertos_sockaddr 	xClient;

socklen_t xSize = sizeof( xClient );

typedef struct TraceStreamPortTCPIP
{
#if (TRC_USE_INTERNAL_BUFFER)
	uint8_t buffer[(TRC_ALIGNED_STREAM_PORT_BUFFER_SIZE)];
#else
	TraceUnsignedBaseType_t buffer[1];
#endif
} TraceStreamPortTCPIP_t;

static TraceStreamPortTCPIP_t* pxStreamPortTCPIP TRC_CFG_RECORDER_DATA_ATTRIBUTE;

static int32_t prvSocketSend(void* pvData, uint32_t uiSize, int32_t* piBytesWritten);
static int32_t prvSocketReceive(void* pvData, uint32_t uiSize, int32_t* piBytesRead);
static int32_t prvSocketInitializeListener(void);
static int32_t prvSocketAccept(void);
static void prvCloseAllSockets(void);

static int32_t prvSocketSend( void* pvData, uint32_t uiSize, int32_t* piBytesWritten )
{
  if (xSocket == NULL)
    return -1;
  
  if (piBytesWritten == (void*)0)
	return -1;
  
	*piBytesWritten =  FreeRTOS_send( xAcceptSocket,
									  ( void * )pvData,
									  uiSize,
									  0);

	if (*piBytesWritten < 0) {
		*piBytesWritten = 0;

		FreeRTOS_closesocket(xAcceptSocket);
		xAcceptSocket = NULL;
		return -1;
	}

	return 0;
}

static int32_t prvSocketReceive( void* pvData, uint32_t uiSize, int32_t* piBytesRead )
{
  if (xSocket == NULL)
    return -1;
  
  if (piBytesRead == (void*)0)
	  return -1;

	*piBytesRead =  FreeRTOS_recv(	xAcceptSocket,
									pvData,
									uiSize,
									0 );
  
	if (*piBytesRead < 0) {
		*piBytesRead = 0;

		FreeRTOS_closesocket(xAcceptSocket);
		xAcceptSocket = NULL;
		return -1;
	}

	return 0;
}

static int32_t prvSocketInitializeListener(void)
{
	static const TickType_t xReceiveTimeOut = pdMS_TO_TICKS( 0 );
	const BaseType_t xBacklog = 20;

	if (xSocket != NULL)
		return 0;
  
	// Attempt to open the socket.
	xSocket = FreeRTOS_socket(FREERTOS_AF_INET, 					// IPv4 アドレスでソケット作成
							FREERTOS_SOCK_STREAM, 				// TCP ソケット指定
							FREERTOS_IPPROTO_TCP );				// TCP ソケット時指定

	// Check the socket was created.
	configASSERT( xSocket != FREERTOS_INVALID_SOCKET );

	// Set a time out so accept() will just wait for a connection.
	FreeRTOS_setsockopt( xSocket,
						 0,
						 FREERTOS_SO_RCVTIMEO,
						 &xReceiveTimeOut,
						 sizeof( xReceiveTimeOut ) );

	// Set the listening port
	xBindAddress.sin_port = FreeRTOS_htons(TRC_CFG_STREAM_PORT_TCPIP_PORT);
//	xBindAddress.sin_family = AF_INET;
//	xBindAddress.sin_addr.s_addr = INADDR_ANY;

	// Bind the socket to the port that the client RTOS task will send to.
	FreeRTOS_bind( xSocket, &xBindAddress, sizeof( xBindAddress ) );

	// Set the socket into a listening state so it can accept connections.
	FreeRTOS_listen( xSocket, xBacklog );

	return 0;
}

static int32_t prvSocketAccept(void)
{
	if (xSocket == NULL)
		return -1;
  
	if (xAcceptSocket != NULL )
		return 0;

	// Wait for incoming connections.
	xAcceptSocket = FreeRTOS_accept( xSocket,
										&xClient,
										&xSize);

	configASSERT( xAcceptSocket != FREERTOS_INVALID_SOCKET );

	return 0;
}

static void prvCloseAllSockets(void)
{
	if (xAcceptSocket != NULL) {
		FreeRTOS_closesocket(xAcceptSocket);
		xAcceptSocket = NULL;
	}
	
	if (xSocket != NULL)
	{
		FreeRTOS_closesocket(xSocket);
		xSocket = NULL;
	}
}
/************** MODIFY THE ABOVE PART TO USE YOUR TPC/IP STACK ****************/

int32_t prvTraceTcpWrite(void* pvData, uint32_t uiSize, int32_t *piBytesWritten)
{
	prvSocketInitializeListener();

	prvSocketAccept();
	
    return prvSocketSend(pvData, uiSize, piBytesWritten);
}

int32_t prvTraceTcpRead(void* pvData, uint32_t uiSize, int32_t *piBytesRead)
{
    prvSocketInitializeListener();
        
    prvSocketAccept();
      
    return prvSocketReceive(pvData, uiSize, piBytesRead);
}

traceResult xTraceStreamPortInitialize(TraceStreamPortBuffer_t* pxBuffer)
{
	TRC_ASSERT_EQUAL_SIZE(TraceStreamPortBuffer_t, TraceStreamPortTCPIP_t);

	if (pxBuffer == 0)
	{
		return TRC_FAIL;
	}

	pxStreamPortTCPIP = (TraceStreamPortTCPIP_t*)pxBuffer;

#if (TRC_USE_INTERNAL_BUFFER == 1)
	return xTraceInternalEventBufferInitialize(pxStreamPortTCPIP->buffer, sizeof(pxStreamPortTCPIP->buffer));
#else
	return TRC_SUCCESS;
#endif
}

traceResult xTraceStreamPortOnTraceEnd(void)
{
	prvCloseAllSockets();

	return TRC_SUCCESS;
}

#endif /*(TRC_CFG_RECORDER_MODE == TRC_RECORDER_MODE_STREAMING)*/

#endif /*(TRC_USE_TRACEALYZER_RECORDER == 1)*/
